# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)


class DynexLisLinkDsxSegmentH(HeaderSegment):
    sender_id = HprimString(5, default="DYNEXRD-LT")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "patients")
    queries = HprimForeign("Q", "queries")
    end = HprimForeign("L", "header", one2one=True, required=True)


class DynexLisLinkDsxSegmentP(Segment):
    sample_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    # reserved = HprimString(6, necessary=True)
    birthdate = HprimDate((8, 1), fmt_date="%Y%m%d")
    sex = HprimString(9)
    location = HprimString((26, 1))

    orders = HprimForeign("O", "orders")


class DynexLisLinkDsxSegmentO(Segment):
    sample_id = HprimString(3)
    test = HprimString((5, 4))
    priority = HprimString(6)
    request_date = HprimDate(7)

    results = HprimForeign("R", "results")


class DynexLisLinkDsxSegmentR(Segment):
    test = HprimString((3, 4))
    value = HprimString(4)
    unit = HprimString(5)
    abnormal_flag = HprimString(7)
    result_status = HprimString(9)
    date_result = HprimDate(13)
    instrument = HprimString(14)


class DynexLisLinkDsxSegmentQ(Segment):
    sample_request = HprimString(3)
    request_info = HprimString(13)


class DynexLisLinkDsxSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):

    SPEC = {"code": "dynex_lislink_dsx", "label": "Dynex LisLink DS2/DSX ASTM"}
    SEGMENT_CLASS_MAP = {
        "H": DynexLisLinkDsxSegmentH,
        "P": DynexLisLinkDsxSegmentP,
        "O": DynexLisLinkDsxSegmentO,
        "R": DynexLisLinkDsxSegmentR,
        "Q": DynexLisLinkDsxSegmentQ,
        "L": DynexLisLinkDsxSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = DynexLisLinkDsxSegmentH(type_segment="H", **kwargs)
        message.end = DynexLisLinkDsxSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            if patient.services:
                service_code = patient.services[0].code
            else:
                service_code = ""
            for sample, priority in patient.samples:
                for sample, priority in patient.samples:
                    for order in sample.orders:
                        seg_p = message.patients.add(
                            sample_id=sample.sample_id,
                            last_name=patient.last_name,
                            first_name=patient.first_name,
                            birthdate=patient.birthdate,
                            sex=patient.sex,
                            location=service_code,
                        )
                        seg_p.orders.add(
                            sample_id=sample.sample_id,
                            test=order.code,
                            priority=priority,
                            request_date=sample.request_date,
                        )

        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.sample_request)
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.sample_id, type="patient"
                    )
                    for seg_r in seg_o.results:
                        flags = []
                        if seg_r.abnormal_flag:
                            flags.append(seg_r.abnormal_flag)
                        sample.results.add(
                            code=seg_r.test,
                            value=seg_r.value.split(" ")[0],
                            flags=flags,
                            unit=seg_r.unit,
                            instrument=seg_r.instrument,
                            date=seg_r.date_result,
                        )
        return worklist
